@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 
 #include "cs_config.h"
@@ -319,7 +320,8 @@ NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned c
   {
     if (buf[l] == '/' || buf[l] == '+' || buf[l] == '=' || buf[l] == '&' || 
 	buf[l] == '"' || buf[l] == '%' || buf[l] == '?' || buf[l] == '#' ||
-	buf[l] == '<' || buf[l] == '>' || buf[l] < 32 || buf[l] > 122)
+	buf[l] == '<' || buf[l] == '>' || buf[l] == '\'' ||
+        buf[l] < 32 || buf[l] > 122)
     {
       nl += 2;
     } 
@@ -358,7 +360,8 @@ NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned c
     {
       if (buf[l] == '/' || buf[l] == '+' || buf[l] == '=' || buf[l] == '&' || 
 	  buf[l] == '"' || buf[l] == '%' || buf[l] == '?' || buf[l] == '#' ||
-	  buf[l] == '<' || buf[l] == '>' || buf[l] < 32 || buf[l] > 122)
+	  buf[l] == '<' || buf[l] == '>' || buf[l] == '\'' ||
+          buf[l] < 32 || buf[l] > 122)
       {
 	match = 1;
       }
@@ -822,7 +825,7 @@ NEOERR *cgi_parse (CGI *cgi)
     {
       char *name;
       err = uListGet(cgi->filenames, uListLength(cgi->filenames)-1, 
-	  (void **)&name);
+	  (void *)&name);
       if (err) return nerr_pass(err);
       err = hdf_set_value (cgi->hdf, "PUT.FileName", name);
       if (err) return nerr_pass(err);
@@ -1015,7 +1018,16 @@ static NEOERR *cgi_compress (STRING *str, char *obuf, int *olen)
 /* This ws strip function is Dave's version, designed to make debug
  * easier, and the output a bit smaller... but not as small as it could
  * be: essentially, it strips all empty lines, all extra space at the
- * end of the line, except in pre/textarea tags */
+ * end of the line, except in pre/textarea tags.
+ *
+ * Ok, expanding to 3 levels:
+ * 0 - No stripping
+ * 1 - Dave's debug stripper (as above)
+ * 2 - strip all extra white space
+ *
+ * We don't currently strip white space in a tag
+ *
+ * */
 
 #if 0
 static void debug_output(char *header, char *s, int n)
@@ -1030,14 +1042,17 @@ static void debug_output(char *header, char *s, int n)
 }
 #endif 
 
-void cgi_html_ws_strip(STRING *str)
+void cgi_html_ws_strip(STRING *str, int level)
 {
   int ws = 0;
-  int seen_nonws = 0;
+  int seen_nonws = level > 1;
   int i, o, l;
   unsigned char *ch;
 
   i = o = 0;
+  if (str->len) {
+    ws = isspace(str->buf[0]);
+  }
   while (i < str->len)
   {
     if (str->buf[i] == '<')
@@ -1113,8 +1128,8 @@ void cgi_html_ws_strip(STRING *str)
        * erasing all blank lines */
       while (o && isspace(str->buf[o-1])) o--;
       str->buf[o++] = str->buf[i++];
-      ws = 0;
-      seen_nonws = 0;
+      ws = level > 1;
+      seen_nonws = level > 1;
     }
     else if (seen_nonws && isspace(str->buf[i]))
     {
@@ -1149,14 +1164,14 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
   int use_gzip = 0;
   int do_debug = 0;
   int do_timefooter = 0;
-  int do_ws_strip = 0;
+  int ws_strip_level = 0;
   char *s, *e;
 
   s = hdf_get_value (cgi->hdf, "Query.debug", NULL);
   e = hdf_get_value (cgi->hdf, "Config.DebugPassword", NULL);
   if (s && e && !strcmp(s, e)) do_debug = 1;
   do_timefooter = hdf_get_int_value (cgi->hdf, "Config.TimeFooter", 1);
-  do_ws_strip = hdf_get_int_value (cgi->hdf, "Config.WhiteSpaceStrip", 1);
+  ws_strip_level = hdf_get_int_value (cgi->hdf, "Config.WhiteSpaceStrip", 1);
 
   dis = ne_timef();
   s = hdf_get_value (cgi->hdf, "cgiout.ContentType", "text/html");
@@ -1243,9 +1258,9 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
       if (err != STATUS_OK) return nerr_pass(err);
     }
 
-    if (do_ws_strip)
+    if (ws_strip_level)
     {
-      cgi_html_ws_strip(str);
+      cgi_html_ws_strip(str, ws_strip_level);
     }
 
     if (do_debug)
@@ -1362,6 +1377,23 @@ NEOERR *cgi_text_html_strfunc(unsigned char *str, unsigned char **ret)
   return nerr_pass(convert_text_html_alloc(str, strlen(str), ret));
 }
 
+NEOERR *cgi_register_strfuncs(CSPARSE *cs)
+{
+  NEOERR *err;
+
+  err = cs_register_strfunc(cs, "url_escape", cgi_url_escape);
+  if (err != STATUS_OK) return nerr_pass(err);
+  err = cs_register_strfunc(cs, "html_escape", cgi_html_escape_strfunc);
+  if (err != STATUS_OK) return nerr_pass(err);
+  err = cs_register_strfunc(cs, "text_html", cgi_text_html_strfunc);
+  if (err != STATUS_OK) return nerr_pass(err);
+  err = cs_register_strfunc(cs, "js_escape", cgi_js_escape);
+  if (err != STATUS_OK) return nerr_pass(err);
+  err = cs_register_strfunc(cs, "html_strip", cgi_html_strip_strfunc);
+  if (err != STATUS_OK) return nerr_pass(err);
+  return STATUS_OK;
+}
+
 NEOERR *cgi_cs_init(CGI *cgi, CSPARSE **cs)
 {
   NEOERR *err;
@@ -1372,15 +1404,7 @@ NEOERR *cgi_cs_init(CGI *cgi, CSPARSE **cs)
   {
     err = cs_init (cs, cgi->hdf);
     if (err != STATUS_OK) break;
-    err = cs_register_strfunc(*cs, "url_escape", cgi_url_escape);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(*cs, "html_escape", cgi_html_escape_strfunc);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(*cs, "text_html", cgi_text_html_strfunc);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(*cs, "js_escape", cgi_js_escape);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(*cs, "html_strip", cgi_html_strip_strfunc);
+    err = cgi_register_strfuncs(*cs);
     if (err != STATUS_OK) break;
   } while (0);
 
@@ -1407,15 +1431,7 @@ NEOERR *cgi_display (CGI *cgi, char *cs_file)
   {
     err = cs_init (&cs, cgi->hdf);
     if (err != STATUS_OK) break;
-    err = cs_register_strfunc(cs, "url_escape", cgi_url_escape);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(cs, "html_escape", cgi_html_escape_strfunc);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(cs, "text_html", cgi_text_html_strfunc);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(cs, "js_escape", cgi_js_escape);
-    if (err != STATUS_OK) break;
-    err = cs_register_strfunc(cs, "html_strip", cgi_html_strip_strfunc);
+    err = cgi_register_strfuncs(cs);
     if (err != STATUS_OK) break;
     err = cs_parse_file (cs, cs_file);
     if (err != STATUS_OK) break;
