@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 
 #ifndef __CGI_H_ 
@@ -452,7 +453,8 @@ NEOERR *cgi_text_html_strfunc(unsigned char *str, unsigned char **ret);
 NEOERR *cgi_html_strip_strfunc(unsigned char *str, unsigned char **ret);
 NEOERR *cgi_html_escape_strfunc(unsigned char *str, unsigned char **ret);
 NEOERR *cgi_js_escape (unsigned char *buf, unsigned char **esc);
-void cgi_html_ws_strip(STRING *str);
+void cgi_html_ws_strip_level(STRING *str, int level);
+NEOERR *cgi_register_strfuncs(CSPARSE *cs);
 
 /* internal use only */
 NEOERR * parse_rfc2388 (CGI *cgi);
