@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 
 #include "cs_config.h"
@@ -435,7 +436,12 @@ static NEOERR *split_and_convert (unsigned char *src, int slen, STRING *out, HTM
 	}
 	err = string_append (out, "\">");
 	if (err != STATUS_OK) break;
-	err = html_escape_alloc(src + x, parts[i].end - x - suffix, &esc);
+        if (opts->link_name) {
+          err = html_escape_alloc(opts->link_name, strlen(opts->link_name), 
+              &esc);
+        } else {
+          err = html_escape_alloc(src + x, parts[i].end - x - suffix, &esc);
+        }
 	if (err != STATUS_OK) break;
 	err = string_append (out, esc);
 	free(esc);
@@ -543,6 +549,7 @@ NEOERR *convert_text_html_alloc_options (unsigned char *src, int slen, unsigned
     opts->newlines_convert = 1;
     opts->longline_width = 75; /* This hasn't been used in a while, actually */
     opts->check_ascii_art = 1;
+    opts->link_name = NULL;
   }
 
   do
