@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 
 #ifndef __HTML_H_ 
@@ -27,6 +28,7 @@ typedef struct _text_html_opts {
     int newlines_convert;
     int longline_width;
     int check_ascii_art;
+    char *link_name;
 } HTML_CONVERT_OPTS;
 
 NEOERR *convert_text_html_alloc (unsigned char *src, int slen, unsigned char **out);
