@@ -1,11 +1,21 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
  *
- * Copyright (C) 2001 by Brandon Long 
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
+ *
+ */
+
+/*
+ * TODO: there is some really ugly pseudo reference counting in here
+ * for allocation of temporary strings (and passing references).  See the alloc
+ * member of various structs for details.  We should move this to an arena
+ * allocator so we can just allocate whenever we need to and just clean up
+ * all the allocation at the end (may require two arenas: one for parese and
+ * one for render)
  */
 
 #include "cs_config.h"
@@ -300,7 +310,6 @@ NEOERR *cs_parse_file (CSPARSE *parse, char *path)
 
 static char *find_context (CSPARSE *parse, int offset, char *buf, size_t blen)
 {
-  NEOERR *err;
   FILE *fp;
   int dump_err = 1;
   char line[256];
@@ -333,8 +342,8 @@ static char *find_context (CSPARSE *parse, int offset, char *buf, size_t blen)
     }
     else
     {
-      err = uListGet(parse->alloc, -1, (void **)&data);
-      if (!err)
+      data = parse->context_string;
+      if (data != NULL)
       {
 	lineno = 1;
 	while (count < offset)
@@ -348,7 +357,6 @@ static char *find_context (CSPARSE *parse, int offset, char *buf, size_t blen)
       }
       else
       {
-	nerr_ignore(&err);
 	if (parse->context)
 	  snprintf (buf, blen, "[%s:%d]", parse->context, offset);
 	else
@@ -404,6 +412,7 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
   char *arg;
   int initial_stack_depth;
   int initial_offset;
+  char *initial_context;
   char tmp[256];
 
   err = uListAppend(parse->alloc, ibuf);
@@ -415,8 +424,10 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
 
   initial_stack_depth = uListLength(parse->stack);
   initial_offset = parse->offset;
+  initial_context = parse->context_string;
 
   parse->offset = 0;
+  parse->context_string = ibuf;
   while (!done)
   {
     /* Stage 1: Find <?cs starter */
@@ -455,7 +466,7 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
 	    if ((Commands[i].has_arg && ((token[n] == ':') || (token[n] == '!')))
 		|| (token[n] == ' ' || token[n] == '\0' || token[n] == '\r' || token[n] == '\n'))
 	    {
-	      err = uListGet (parse->stack, -1, (void **)&entry);
+	      err = uListGet (parse->stack, -1, (void *)&entry);
 	      if (err != STATUS_OK) goto cs_parse_done;
 	      if (!(Commands[i].allowed_state & entry->state))
 	      {
@@ -477,8 +488,10 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
 	      if (err != STATUS_OK) goto cs_parse_done;
 	      if (Commands[i].next_state & ST_POP)
 	      {
-		err = uListPop(parse->stack, (void **)&entry);
+                void *ptr;
+		err = uListPop(parse->stack, &ptr);
 		if (err != STATUS_OK) goto cs_parse_done;
+                entry = (STACK_ENTRY *)ptr;
 		if (entry->next_tree)
 		  parse->current = entry->next_tree;
 		else
@@ -522,7 +535,7 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
   /* Should we check the parse stack here? */
   while (uListLength(parse->stack) > initial_stack_depth)
   {
-    err = uListPop(parse->stack, (void **)&entry);
+    err = uListPop(parse->stack, (void *)&entry);
     if (err != STATUS_OK) goto cs_parse_done;
     if (entry->state & (ST_IF | ST_ELSE))
       return nerr_raise (NERR_PARSE, "%s Non-terminted if clause",
@@ -534,6 +547,7 @@ NEOERR *cs_parse_string (CSPARSE *parse, char *ibuf, size_t ibuf_len)
 
 cs_parse_done:
   parse->offset = initial_offset;
+  parse->context_string = initial_context;
   return nerr_pass(err);
 }
 
@@ -571,11 +585,11 @@ static HDF *var_lookup_obj (CSPARSE *parse, char *name)
   {
     if (c == NULL)
     {
-      return map->value.h;
+      return map->h;
     }
     else
     {
-      return hdf_get_obj (map->value.h, c+1);
+      return hdf_get_obj (map->h, c+1);
     }
   }
   return hdf_get_obj (parse->hdf, name);
@@ -599,12 +613,12 @@ static NEOERR *var_set_value (CSPARSE *parse, char *name, char *value)
       {
 	if (c == NULL)
 	{
-	  return nerr_pass (hdf_set_value (map->value.h, NULL, value));
+	  return nerr_pass (hdf_set_value (map->h, NULL, value));
 	}
 	else
 	{
 	  *c = '.';
-	  return nerr_pass (hdf_set_value (map->value.h, c+1, value));
+	  return nerr_pass (hdf_set_value (map->h, c+1, value));
 	}
       }
       else
@@ -615,13 +629,13 @@ static NEOERR *var_set_value (CSPARSE *parse, char *name, char *value)
 	  /* If this is a string, it might be what we're setting,
 	   * ie <?cs set:value = value ?>
 	   */
-	  if (map->type == CS_TYPE_STRING && map->alloc)
-	    tmp = map->value.s;
+	  if (map->type == CS_TYPE_STRING && map->map_alloc)
+	    tmp = map->s;
 	  map->type = CS_TYPE_STRING;
-	  map->alloc = 1;
-	  map->value.s = strdup(value);
+	  map->map_alloc = 1;
+	  map->s = strdup(value);
 	  if (tmp != NULL) free(tmp);
-	  if (map->value.s == NULL && value != NULL)
+	  if (map->s == NULL && value != NULL)
 	    return nerr_raise(NERR_NOMEM, 
 		"Unable to allocate memory to set var");
 
@@ -651,11 +665,11 @@ static char *var_lookup (CSPARSE *parse, char *name)
     {
       if (c == NULL)
       {
-	return hdf_obj_value (map->value.h);
+	return hdf_obj_value (map->h);
       }
       else
       {
-	return hdf_get_value (map->value.h, c+1, NULL);
+	return hdf_get_value (map->h, c+1, NULL);
       }
     }
     /* Hmm, if c != NULL, they are asking for a sub member of something
@@ -665,13 +679,16 @@ static char *var_lookup (CSPARSE *parse, char *name)
      * string that will be deleted... where is it used? */
     else if (map->type == CS_TYPE_STRING)
     {
-      return map->value.s;
+      return map->s;
     }
     else if (map->type == CS_TYPE_NUM)
     {
-      static char buf[40];
-      snprintf (buf, sizeof(buf), "%ld", map->value.n);
-      return buf;
+      char buf[40];
+      if (map->s) return map->s;
+      snprintf (buf, sizeof(buf), "%ld", map->n);
+      map->s = strdup(buf);
+      map->map_alloc = 1;
+      return map->s;
     }
   }
   return hdf_get_value (parse->hdf, name, NULL);
@@ -885,12 +902,12 @@ CSTOKEN_TYPE OperatorOrder[] = {
   CS_OP_COMMA,
   CS_OP_OR,
   CS_OP_AND,
-  CS_OP_NOT | CS_OP_EXISTS,
   CS_OP_EQUAL | CS_OP_NEQUAL,
   CS_OP_GT | CS_OP_GTE | CS_OP_LT | CS_OP_LTE,
   CS_OP_ADD | CS_OP_SUB, 
-  CS_OP_MULT | CS_OP_DIV | CS_OP_MOD | CS_OP_DOT,
-  CS_OP_LBRACKET,
+  CS_OP_MULT | CS_OP_DIV | CS_OP_MOD,
+  CS_OP_NOT | CS_OP_EXISTS,
+  CS_OP_LBRACKET | CS_OP_DOT,
   0
 };
 
@@ -962,7 +979,7 @@ static NEOERR *parse_expr2 (CSPARSE *parse, CSTOKEN *tokens, int ntokens, int lv
   NEOERR *err;
   char tmp[256];
   char tmp2[256];
-  int x, op = 0;
+  int x, op;
   int m;
 
 #if DEBUG_EXPR_PARSE
@@ -1016,6 +1033,7 @@ static NEOERR *parse_expr2 (CSPARSE *parse, CSTOKEN *tokens, int ntokens, int lv
   }
   */
 
+  op = 0;
   while (OperatorOrder[op])
   {
     x = ntokens-1;
@@ -1828,6 +1846,9 @@ static NEOERR *eval_expr (CSPARSE *parse, CSARG *expr, CSARG *result)
   if (result == NULL)
     return nerr_raise (NERR_ASSERT, "result is NULL");
 
+  arg1.alloc = 0;
+  arg2.alloc = 0;
+
 #if DEBUG_EXPR_EVAL
   _depth++;
   expand_arg(parse, _depth, "expr", expr);
@@ -1845,192 +1866,175 @@ static NEOERR *eval_expr (CSPARSE *parse, CSARG *expr, CSARG *result)
 #endif
     return STATUS_OK;
   }
-  else
-  {
-    err = eval_expr (parse, expr->expr1, &arg1);
-    if (err) return nerr_pass(err);
-#if DEBUG_EXPR_EVAL
-    expand_arg(parse, _depth, "arg1", &arg1);
-#endif
-    if (expr->op_type & CS_TYPE_FUNCTION)
-    {
-      if (expr->function == NULL || expr->function->function == NULL)
-	return nerr_raise(NERR_ASSERT, 
-	    "Function is NULL in attempt to evaluate function call %s", 
-	    (expr->function) ? expr->function->name : "");
 
-      err = expr->function->function(parse, expr->function, &arg1, result);
-      if (err) return nerr_pass(err);
-    }
-    else if (expr->op_type & CS_OPS_UNARY)
-    {
-      result->op_type = CS_TYPE_NUM;
-      switch (expr->op_type) {
-	case CS_OP_NOT:
-	  result->n = arg_eval_bool(parse, &arg1) ? 0 : 1;
-	  /*
-	  if (arg1.op_type & CS_TYPE_VAR)
-	  {
-	    / * This case is a "not exist" test * /
-	    s1 = arg_eval (parse, &arg1);
-	    if (s1 == NULL || *s1 == '\0')
-	      result->n = 1;
-	    else
-	      result->n = 0;
-	  }
-	  else 
-	  {
-	    result->n = arg_eval_num (parse, &arg1);
-	    result->n = result->n ? 0 : 1;
-	  }
-	  */
-	  break;
-	case CS_OP_EXISTS:
-	  if (arg1.op_type & (CS_TYPE_VAR | CS_TYPE_VAR_NUM))
-	  {
-	    s1 = arg_eval (parse, &arg1);
-	    if (s1 == NULL || *s1 == '\0')
-	      result->n = 0;
-	    else
-	      result->n = 1;
-	  }
-	  else
-	  {
-	    /* All numbers/strings exist */
-	    result->n = 1;
-	  }
-	  break;
-	case CS_OP_NUM:
-	  result->n = arg_eval_num (parse, &arg1);
-	  break;
-	default:
-	  result->n = 0;
-	  ne_warn ("Unsupported op %s in eval_expr", expand_token_type(expr->op_type, 1));
-	  break;
-      }
-    }
-    else if (expr->op_type == CS_OP_COMMA)
-    {
-      /* The comma operator, like in C, we return the value of the right
-       * most argument, in this case that's expr1, but we still need to
-       * evaluate the other stuff */
-      if (expr->next)
-      {
-	err = eval_expr (parse, expr->next, &arg2);
+  err = eval_expr (parse, expr->expr1, &arg1);
+  if (err) return nerr_pass(err);
 #if DEBUG_EXPR_EVAL
-	expand_arg(parse, _depth, "arg2", &arg2);
+  expand_arg(parse, _depth, "arg1", &arg1);
 #endif
-	if (err) return nerr_pass(err);
-	if (arg2.alloc) free(arg2.s);
-      }
-      *result = arg1;
-      /* we transfer ownership of the string here.. ugh */
-      if (arg1.alloc) arg1.alloc = 0;
-#if DEBUG_EXPR_EVAL
-      expand_arg(parse, _depth, "result", result);
-      _depth--;
-#endif
-      return STATUS_OK;
+  if (expr->op_type & CS_TYPE_FUNCTION)
+  {
+    if (expr->function == NULL || expr->function->function == NULL)
+      return nerr_raise(NERR_ASSERT, 
+          "Function is NULL in attempt to evaluate function call %s", 
+          (expr->function) ? expr->function->name : "");
+
+    err = expr->function->function(parse, expr->function, &arg1, result);
+    if (err) return nerr_pass(err);
+  }
+  else if (expr->op_type & CS_OPS_UNARY)
+  {
+    result->op_type = CS_TYPE_NUM;
+    switch (expr->op_type) {
+      case CS_OP_NOT:
+        result->n = arg_eval_bool(parse, &arg1) ? 0 : 1;
+        break;
+      case CS_OP_EXISTS:
+        if (arg1.op_type & (CS_TYPE_VAR | CS_TYPE_VAR_NUM))
+        {
+          s1 = arg_eval (parse, &arg1);
+          if (s1 == NULL)
+            result->n = 0;
+          else
+            result->n = 1;
+        }
+        else
+        {
+          /* All numbers/strings exist */
+          result->n = 1;
+        }
+        break;
+      case CS_OP_NUM:
+        result->n = arg_eval_num (parse, &arg1);
+        break;
+      default:
+        result->n = 0;
+        ne_warn ("Unsupported op %s in eval_expr", expand_token_type(expr->op_type, 1));
+        break;
     }
-    else
+  }
+  else if (expr->op_type == CS_OP_COMMA)
+  {
+    /* The comma operator, like in C, we return the value of the right
+     * most argument, in this case that's expr1, but we still need to
+     * evaluate the other stuff */
+    if (expr->next)
     {
-      err = eval_expr (parse, expr->expr2, &arg2);
+      err = eval_expr (parse, expr->next, &arg2);
 #if DEBUG_EXPR_EVAL
       expand_arg(parse, _depth, "arg2", &arg2);
 #endif
       if (err) return nerr_pass(err);
-
-      if (expr->op_type == CS_OP_LBRACKET)
-      {
-	/* the bracket op is essentially hdf array lookups, which just
-	 * means appending the value of arg2, .0 */
-	result->op_type = CS_TYPE_VAR;
-	result->alloc = 1;
-	if (arg2.op_type & (CS_TYPE_VAR_NUM | CS_TYPE_NUM))
-	{
-	  n2 = arg_eval_num (parse, &arg2);
-	  result->s = sprintf_alloc("%s.%d", arg1.s, n2);
-	  if (result->s == NULL)
-	    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %d", arg1.s, n2);
-	}
-	else
-	{
-	  s2 = arg_eval (parse, &arg2);
-	  if (s2 && s2[0])
-	  {
-	    result->s = sprintf_alloc("%s.%s", arg1.s, s2);
-	    if (result->s == NULL)
-	      return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, s2);
-	  }
-	  else
-	  {
-	    /* if s2 doesn't match anything, then the whole thing is empty */
-	    result->s = "";
-	    result->alloc = 0;
-	  }
-	}
-      }
-      else if (expr->op_type == CS_OP_DOT)
-      {
-	/* the dot op is essentially extending the hdf name, which just
-	 * means appending the string .0 */
-	result->op_type = CS_TYPE_VAR;
-	result->alloc = 1;
-	if (arg2.op_type & CS_TYPES_VAR)
-	{
-	  result->s = sprintf_alloc("%s.%s", arg1.s, arg2.s);
-	  if (result->s == NULL)
-	    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, arg2.s);
-	}
-	else
-	{
-	  if (arg2.op_type & CS_TYPE_NUM)
-	  {
-	    n2 = arg_eval_num (parse, &arg2);
-	    result->s = sprintf_alloc("%s.%d", arg1.s, n2);
-	    if (result->s == NULL)
-	      return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %d", arg1.s, n2);
-	  }
-	  else
-	  {
-	    s2 = arg_eval (parse, &arg2);
-	    if (s2 && s2[0])
-	    {
-	      result->s = sprintf_alloc("%s.%s", arg1.s, s2);
-	      if (result->s == NULL)
-		return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, s2);
-	    }
-	    else
-	    {
-	      /* if s2 doesn't match anything, then the whole thing is empty */
-	      result->s = "";
-	      result->alloc = 0;
-	    }
-	  }
-	}
-      }
-      else if (expr->op_type & (CS_OP_AND | CS_OP_OR))
-      {
-	/* eval as bool */
-	err = eval_expr_bool (parse, &arg1, &arg2, expr->op_type, result);
-      }
-      else if ((arg1.op_type & (CS_TYPE_NUM | CS_TYPE_VAR_NUM)) ||
-	  (arg2.op_type & (CS_TYPE_NUM | CS_TYPE_VAR_NUM)) ||
-	  (expr->op_type & (CS_OP_AND | CS_OP_OR | CS_OP_SUB | CS_OP_MULT | CS_OP_DIV | CS_OP_MOD | CS_OP_GT | CS_OP_GTE | CS_OP_LT | CS_OP_LTE)))
-      {
-	/* eval as num */
-	err = eval_expr_num(parse, &arg1, &arg2, expr->op_type, result);
-
-      }
-      else /* eval as string */
-      {
-	err = eval_expr_string(parse, &arg1, &arg2, expr->op_type, result);
-      }
-
-      if (arg1.alloc) free(arg1.s);
       if (arg2.alloc) free(arg2.s);
     }
+    *result = arg1;
+    /* we transfer ownership of the string here.. ugh */
+    if (arg1.alloc) arg1.alloc = 0;
+#if DEBUG_EXPR_EVAL
+    expand_arg(parse, _depth, "result", result);
+    _depth--;
+#endif
+    return STATUS_OK;
   }
+  else
+  {
+    err = eval_expr (parse, expr->expr2, &arg2);
+#if DEBUG_EXPR_EVAL
+    expand_arg(parse, _depth, "arg2", &arg2);
+#endif
+    if (err) return nerr_pass(err);
+
+    if (expr->op_type == CS_OP_LBRACKET)
+    {
+      /* the bracket op is essentially hdf array lookups, which just
+       * means appending the value of arg2, .0 */
+      result->op_type = CS_TYPE_VAR;
+      result->alloc = 1;
+      if (arg2.op_type & (CS_TYPE_VAR_NUM | CS_TYPE_NUM))
+      {
+        n2 = arg_eval_num (parse, &arg2);
+        result->s = sprintf_alloc("%s.%d", arg1.s, n2);
+        if (result->s == NULL)
+          return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %d", arg1.s, n2);
+      }
+      else
+      {
+        s2 = arg_eval (parse, &arg2);
+        if (s2 && s2[0])
+        {
+          result->s = sprintf_alloc("%s.%s", arg1.s, s2);
+          if (result->s == NULL)
+            return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, s2);
+        }
+        else
+        {
+          /* if s2 doesn't match anything, then the whole thing is empty */
+          result->s = "";
+          result->alloc = 0;
+        }
+      }
+    }
+    else if (expr->op_type == CS_OP_DOT)
+    {
+      /* the dot op is essentially extending the hdf name, which just
+       * means appending the string .0 */
+      result->op_type = CS_TYPE_VAR;
+      result->alloc = 1;
+      if (arg2.op_type & CS_TYPES_VAR)
+      {
+        result->s = sprintf_alloc("%s.%s", arg1.s, arg2.s);
+        if (result->s == NULL)
+          return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, arg2.s);
+      }
+      else
+      {
+        if (arg2.op_type & CS_TYPE_NUM)
+        {
+          n2 = arg_eval_num (parse, &arg2);
+          result->s = sprintf_alloc("%s.%d", arg1.s, n2);
+          if (result->s == NULL)
+            return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %d", arg1.s, n2);
+        }
+        else
+        {
+          s2 = arg_eval (parse, &arg2);
+          if (s2 && s2[0])
+          {
+            result->s = sprintf_alloc("%s.%s", arg1.s, s2);
+            if (result->s == NULL)
+              return nerr_raise (NERR_NOMEM, "Unable to allocate memory to concatenate varnames in expression: %s + %s", arg1.s, s2);
+          }
+          else
+          {
+            /* if s2 doesn't match anything, then the whole thing is empty */
+            result->s = "";
+            result->alloc = 0;
+          }
+        }
+      }
+    }
+    else if (expr->op_type & (CS_OP_AND | CS_OP_OR))
+    {
+      /* eval as bool */
+      err = eval_expr_bool (parse, &arg1, &arg2, expr->op_type, result);
+    }
+    else if ((arg1.op_type & (CS_TYPE_NUM | CS_TYPE_VAR_NUM)) ||
+        (arg2.op_type & (CS_TYPE_NUM | CS_TYPE_VAR_NUM)) ||
+        (expr->op_type & (CS_OP_AND | CS_OP_OR | CS_OP_SUB | CS_OP_MULT | CS_OP_DIV | CS_OP_MOD | CS_OP_GT | CS_OP_GTE | CS_OP_LT | CS_OP_LTE)))
+    {
+      /* eval as num */
+      err = eval_expr_num(parse, &arg1, &arg2, expr->op_type, result);
+
+    }
+    else /* eval as string */
+    {
+      err = eval_expr_string(parse, &arg1, &arg2, expr->op_type, result);
+    }
+
+  }
+  if (arg1.alloc) free(arg1.s);
+  if (arg2.alloc) free(arg2.s);
+
 #if DEBUG_EXPR_EVAL
   expand_arg(parse, _depth, "result", result);
   _depth--;
@@ -2224,28 +2228,6 @@ static NEOERR *if_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   err = eval_expr(parse, &(node->arg1), &val);
   if (err) return nerr_pass (err);
   eval_true = arg_eval_bool(parse, &val);
-  /*
-  if (val.op_type & (CS_TYPE_NUM | CS_TYPE_VAR_NUM))
-    eval_true = arg_eval_num (parse, &val);
-  else
-  {
-    char *s;
-    BOOL not = FALSE;
-    if (val.op_type & CS_OP_NOT)
-    {
-      not = TRUE;
-      val.op_type &= ~CS_OP_NOT;
-    }
-    s = arg_eval (parse, &val);
-    if (s == NULL || *s == '\0')
-      eval_true = 0;
-    else
-      eval_true = 1;
-
-    if (not == TRUE)
-      eval_true = !eval_true;
-  }
-  */
   if (val.alloc) free(val.s);
 
   if (eval_true)
@@ -2266,7 +2248,7 @@ static NEOERR *else_parse (CSPARSE *parse, int cmd, char *arg)
   STACK_ENTRY *entry;
 
   /* ne_warn ("else"); */
-  err = uListGet (parse->stack, -1, (void **)&entry);
+  err = uListGet (parse->stack, -1, (void *)&entry);
   if (err != STATUS_OK) return nerr_pass(err);
 
   parse->next = &(entry->tree->case_1);
@@ -2280,7 +2262,7 @@ static NEOERR *elif_parse (CSPARSE *parse, int cmd, char *arg)
   STACK_ENTRY *entry;
 
   /* ne_warn ("elif: %s", arg); */
-  err = uListGet (parse->stack, -1, (void **)&entry);
+  err = uListGet (parse->stack, -1, (void *)&entry);
   if (err != STATUS_OK) return nerr_pass(err);
 
   if (entry->next_tree == NULL)
@@ -2299,7 +2281,7 @@ static NEOERR *endif_parse (CSPARSE *parse, int cmd, char *arg)
   STACK_ENTRY *entry;
 
   /* ne_warn ("endif"); */
-  err = uListGet (parse->stack, -1, (void **)&entry);
+  err = uListGet (parse->stack, -1, (void *)&entry);
   if (err != STATUS_OK) return nerr_pass(err);
 
   if (entry->next_tree)
@@ -2384,6 +2366,8 @@ static NEOERR *each_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   CSARG val;
   HDF *var, *child;
 
+  memset(&each_map, 0, sizeof(each_map));
+
   err = eval_expr(parse, &(node->arg2), &val);
   if (err) return nerr_pass(err);
 
@@ -2404,8 +2388,12 @@ static NEOERR *each_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
 	child = hdf_obj_child (var);
 	while (child != NULL)
 	{
-	  each_map.value.h = child;
+	  each_map.h = child;
 	  err = render_node (parse, node->case_0);
+          if (each_map.map_alloc) {
+            free(each_map.s);
+            each_map.s = NULL;
+          }
 	  if (err != STATUS_OK) break;
 	  child = hdf_obj_next (child);
 	}
@@ -2429,6 +2417,8 @@ static NEOERR *with_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   CSARG val;
   HDF *var;
 
+  memset(&with_map, 0, sizeof(with_map));
+
   err = eval_expr(parse, &(node->arg2), &val);
   if (err) return nerr_pass(err);
 
@@ -2442,10 +2432,11 @@ static NEOERR *with_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
       with_map.type = CS_TYPE_VAR;
       with_map.name = node->arg1.s;
       with_map.next = parse->locals;
-      with_map.value.h = var;
+      with_map.h = var;
       parse->locals = &with_map;
       err = render_node (parse, node->case_0);
       /* Remove local map */
+      if (with_map.map_alloc) free(with_map.s);
       parse->locals = with_map.next;
     }
   } /* else WARNING */
@@ -2459,7 +2450,7 @@ static NEOERR *end_parse (CSPARSE *parse, int cmd, char *arg)
   NEOERR *err;
   STACK_ENTRY *entry;
 
-  err = uListGet (parse->stack, -1, (void **)&entry);
+  err = uListGet (parse->stack, -1, (void *)&entry);
   if (err != STATUS_OK) return nerr_pass(err);
 
   parse->next = &(entry->tree->next);
@@ -2494,6 +2485,7 @@ static NEOERR *include_parse (CSPARSE *parse, int cmd, char *arg)
   {
     nerr_handle(&err, NERR_NOT_FOUND);
   }
+  if (val.alloc) free(val.s);
 
   return nerr_pass (err);
 }
@@ -2802,14 +2794,14 @@ static NEOERR *call_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
     if (err) break;
     if (val.op_type & CS_TYPE_STRING)
     {
-      map->value.s = val.s;
+      map->s = val.s;
       map->type = val.op_type;
-      map->alloc = val.alloc;
+      map->map_alloc = val.alloc;
       val.alloc = 0;
     }
     else if (val.op_type & CS_TYPE_NUM)
     {
-      map->value.n = val.n;
+      map->n = val.n;
       map->type = CS_TYPE_NUM;
     }
     else if (val.op_type & (CS_TYPE_VAR | CS_TYPE_VAR_NUM))
@@ -2823,19 +2815,19 @@ static NEOERR *call_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
 	 * number... then copy  */
 	if (lmap->type == CS_TYPE_NUM)
 	{
-	  map->value.n = lmap->value.n;
+	  map->n = lmap->n;
 	  map->type = lmap->type;
 	}
 	else
 	{
-	  map->value.s = lmap->value.s;
+	  map->s = lmap->s;
 	  map->type = lmap->type;
 	}
       }
       else
       {
 	var = var_lookup_obj (parse, val.s);
-	map->value.h = var;
+	map->h = var;
 	map->type = CS_TYPE_VAR;
       }
     }
@@ -2859,7 +2851,7 @@ static NEOERR *call_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   }
   for (x = 0; x < macro->n_args; x++)
   {
-    if (call_map[x].alloc) free(call_map[x].value.s);
+    if (call_map[x].map_alloc) free(call_map[x].s);
   }
   free (call_map);
 
@@ -3079,6 +3071,8 @@ static NEOERR *loop_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   CSARG *carg;
   CSARG val;
 
+  memset(&each_map, 0, sizeof(each_map));
+
   carg = node->vargs;
   if (carg == NULL) return nerr_raise (NERR_ASSERT, "No arguments in loop eval?");
   err = eval_expr(parse, carg, &val);
@@ -3127,8 +3121,12 @@ static NEOERR *loop_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
     var = start;
     for (x = 0, var = start; x < iter; x++, var += step)
     {
-      each_map.value.n = var;
+      each_map.n = var;
       err = render_node (parse, node->case_0);
+      if (each_map.map_alloc) {
+        free(each_map.s);
+        each_map.s = NULL;
+      }
       if (err != STATUS_OK) break;
     } 
 
