@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 
 #include <Python.h>
@@ -412,7 +413,7 @@ static PyObject * p_text_html (PyObject *self, PyObject *args, PyObject *keywds)
   PyObject *rv;
   int len;
   HTML_CONVERT_OPTS opts;
-  static char *kwlist[] = {"text", "bounce_url", "url_class", "url_target", "mailto_class", "long_lines", "space_convert", "newlines_convert", "longline_width", "check_ascii_art", NULL};
+  static char *kwlist[] = {"text", "bounce_url", "url_class", "url_target", "mailto_class", "long_lines", "space_convert", "newlines_convert", "longline_width", "check_ascii_art", "link_name", NULL};
 
   /* These defaults all come from the old version */
   opts.bounce_url = NULL;
@@ -424,12 +425,13 @@ static PyObject * p_text_html (PyObject *self, PyObject *args, PyObject *keywds)
   opts.newlines_convert = 1;
   opts.longline_width = 75; /* This hasn't been used in a while, actually */
   opts.check_ascii_art = 1;
+  opts.link_name = NULL;
 
-  if (!PyArg_ParseTupleAndKeywords(args, keywds, "s#|ssssiiiii:text2html(text)", 
+  if (!PyArg_ParseTupleAndKeywords(args, keywds, "s#|ssssiiiiis:text2html(text)", 
 	kwlist, 
 	&s, &len, &(opts.bounce_url), &(opts.url_class), &(opts.url_target), 
 	&(opts.mailto_class), &(opts.long_lines), &(opts.space_convert), 
-	&(opts.newlines_convert), &(opts.longline_width), &(opts.check_ascii_art)))
+	&(opts.newlines_convert), &(opts.longline_width), &(opts.check_ascii_art), &(opts.link_name)))
     return NULL;
 
   err = convert_text_html_alloc_options (s, len, &esc, &opts);
