@@ -1,11 +1,12 @@
 /*
- * Neotonic ClearSilver Templating System
+ * Copyright 2001-2004 Brandon Long
+ * All Rights Reserved.
  *
- * This code is made available under the terms of the 
- * Neotonic ClearSilver License.
- * http://www.neotonic.com/clearsilver/license.hdf
+ * ClearSilver Templating System
+ *
+ * This code is made available under the terms of the ClearSilver License.
+ * http://www.clearsilver.net/license.hdf
  *
- * Copyright (C) 2001 by Brandon Long
  */
 /*
  * The wdb is a wrapper around the sleepycat db library which adds
@@ -902,7 +903,7 @@ NEOERR *wdb_column_update (WDB *wdb, char *oldkey, char *newkey)
   len = uListLength(wdb->cols_l);
   for (x = 0; x < len; x++)
   {
-    err = uListGet (wdb->cols_l, x, (void **)&vcol);
+    err = uListGet (wdb->cols_l, x, (void *)&vcol);
     if (err) return nerr_pass(err);
     if (!strcmp(vcol->name, oldkey))
     {
